;;;
;;;Part of: Vicare Scheme
;;;Contents: errno constants and macros
;;;Date: Tue Feb  7, 2012
;;;
;;;Abstract
;;;
;;;
;;;
;;;Copyright (C) 2012, 2013 Marco Maggi <marco.maggi-ipsu@poste.it>
;;;
;;;This program is free software:  you can redistribute it and/or modify
;;;it under the terms of the  GNU General Public License as published by
;;;the Free Software Foundation, either version 3 of the License, or (at
;;;your option) any later version.
;;;
;;;This program is  distributed in the hope that it  will be useful, but
;;;WITHOUT  ANY   WARRANTY;  without   even  the  implied   warranty  of
;;;MERCHANTABILITY  or FITNESS FOR  A PARTICULAR  PURPOSE.  See  the GNU
;;;General Public License for more details.
;;;
;;;You should  have received  a copy of  the GNU General  Public License
;;;along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;;


#!r6rs
(library (vicare platform errno)
  (export
    errno-code		case-errno

    E2BIG		EACCES		EADDRINUSE
    EADDRNOTAVAIL	EADV		EAFNOSUPPORT
    EAGAIN		EALREADY	EBADE
    EBADF		EBADFD		EBADMSG
    EBADR		EBADRQC		EBADSLT
    EBFONT		EBUSY		ECANCELED
    ECHILD		ECHRNG		ECOMM
    ECONNABORTED	ECONNREFUSED	ECONNRESET
    EDEADLK		EDEADLOCK	EDESTADDRREQ
    EDOM		EDOTDOT		EDQUOT
    EEXIST		EFAULT		EFBIG
    EHOSTDOWN		EHOSTUNREACH	EIDRM
    EILSEQ		EINPROGRESS	EINTR
    EINVAL		EIO		EISCONN
    EISDIR		EISNAM		EKEYEXPIRED
    EKEYREJECTED	EKEYREVOKED	EL2HLT
    EL2NSYNC		EL3HLT		EL3RST
    ELIBACC		ELIBBAD		ELIBEXEC
    ELIBMAX		ELIBSCN		ELNRNG
    ELOOP		EMEDIUMTYPE	EMFILE
    EMLINK		EMSGSIZE	EMULTIHOP
    ENAMETOOLONG	ENAVAIL		ENETDOWN
    ENETRESET		ENETUNREACH	ENFILE
    ENOANO		ENOBUFS		ENOCSI
    ENODATA		ENODEV		ENOENT
    ENOEXEC		ENOKEY		ENOLCK
    ENOLINK		ENOMEDIUM	ENOMEM
    ENOMSG		ENONET		ENOPKG
    ENOPROTOOPT		ENOSPC		ENOSR
    ENOSTR		ENOSYS		ENOTBLK
    ENOTCONN		ENOTDIR		ENOTEMPTY
    ENOTNAM		ENOTRECOVERABLE	ENOTSOCK
    ENOTTY		ENOTUNIQ	ENXIO
    EOPNOTSUPP		EOVERFLOW	EOWNERDEAD
    EPERM		EPFNOSUPPORT	EPIPE
    EPROTO		EPROTONOSUPPORT	EPROTOTYPE
    ERANGE		EREMCHG		EREMOTE
    EREMOTEIO		ERESTART	EROFS
    ESHUTDOWN		ESOCKTNOSUPPORT	ESPIPE
    ESRCH		ESRMNT		ESTALE
    ESTRPIPE		ETIME		ETIMEDOUT
    ETOOMANYREFS	ETXTBSY		EUCLEAN
    EUNATCH		EUSERS		EWOULDBLOCK
    EXDEV		EXFULL)
  (import (vicare)
    (only (vicare system $fx)
	  $fx=))


;;;; helpers

(define-syntax neg
  (lambda (stx)
    (syntax-case stx ()
      ((_ ?code)
       (let ((N (syntax->datum #'?code)))
	 (if (number? N)
	     (- N)
	   #t))))))


;;;; errno codes

(define-inline-constant E2BIG		(neg 7))
(define-inline-constant EACCES		(neg 13))
(define-inline-constant EADDRINUSE	(neg 98))
(define-inline-constant EADDRNOTAVAIL	(neg 99))
(define-inline-constant EADV		(neg 68))
(define-inline-constant EAFNOSUPPORT	(neg 97))
(define-inline-constant EAGAIN		(neg 11))
(define-inline-constant EALREADY	(neg 114))
(define-inline-constant EBADE		(neg 52))
(define-inline-constant EBADF		(neg 9))
(define-inline-constant EBADFD		(neg 77))
(define-inline-constant EBADMSG		(neg 74))
(define-inline-constant EBADR		(neg 53))
(define-inline-constant EBADRQC		(neg 56))
(define-inline-constant EBADSLT		(neg 57))
(define-inline-constant EBFONT		(neg 59))
(define-inline-constant EBUSY		(neg 16))
(define-inline-constant ECANCELED	(neg 125))
(define-inline-constant ECHILD		(neg 10))
(define-inline-constant ECHRNG		(neg 44))
(define-inline-constant ECOMM		(neg 70))
(define-inline-constant ECONNABORTED	(neg 103))
(define-inline-constant ECONNREFUSED	(neg 111))
(define-inline-constant ECONNRESET	(neg 104))
(define-inline-constant EDEADLK		(neg 35))
(define-inline-constant EDEADLOCK	(neg 35))
(define-inline-constant EDESTADDRREQ	(neg 89))
(define-inline-constant EDOM		(neg 33))
(define-inline-constant EDOTDOT		(neg 73))
(define-inline-constant EDQUOT		(neg 122))
(define-inline-constant EEXIST		(neg 17))
(define-inline-constant EFAULT		(neg 14))
(define-inline-constant EFBIG		(neg 27))
(define-inline-constant EHOSTDOWN	(neg 112))
(define-inline-constant EHOSTUNREACH	(neg 113))
(define-inline-constant EIDRM		(neg 43))
(define-inline-constant EILSEQ		(neg 84))
(define-inline-constant EINPROGRESS	(neg 115))
(define-inline-constant EINTR		(neg 4))
(define-inline-constant EINVAL		(neg 22))
(define-inline-constant EIO		(neg 5))
(define-inline-constant EISCONN		(neg 106))
(define-inline-constant EISDIR		(neg 21))
(define-inline-constant EISNAM		(neg 120))
(define-inline-constant EKEYEXPIRED	(neg 127))
(define-inline-constant EKEYREJECTED	(neg 129))
(define-inline-constant EKEYREVOKED	(neg 128))
(define-inline-constant EL2HLT		(neg 51))
(define-inline-constant EL2NSYNC	(neg 45))
(define-inline-constant EL3HLT		(neg 46))
(define-inline-constant EL3RST		(neg 47))
(define-inline-constant ELIBACC		(neg 79))
(define-inline-constant ELIBBAD		(neg 80))
(define-inline-constant ELIBEXEC	(neg 83))
(define-inline-constant ELIBMAX		(neg 82))
(define-inline-constant ELIBSCN		(neg 81))
(define-inline-constant ELNRNG		(neg 48))
(define-inline-constant ELOOP		(neg 40))
(define-inline-constant EMEDIUMTYPE	(neg 124))
(define-inline-constant EMFILE		(neg 24))
(define-inline-constant EMLINK		(neg 31))
(define-inline-constant EMSGSIZE	(neg 90))
(define-inline-constant EMULTIHOP	(neg 72))
(define-inline-constant ENAMETOOLONG	(neg 36))
(define-inline-constant ENAVAIL		(neg 119))
(define-inline-constant ENETDOWN	(neg 100))
(define-inline-constant ENETRESET	(neg 102))
(define-inline-constant ENETUNREACH	(neg 101))
(define-inline-constant ENFILE		(neg 23))
(define-inline-constant ENOANO		(neg 55))
(define-inline-constant ENOBUFS		(neg 105))
(define-inline-constant ENOCSI		(neg 50))
(define-inline-constant ENODATA		(neg 61))
(define-inline-constant ENODEV		(neg 19))
(define-inline-constant ENOENT		(neg 2))
(define-inline-constant ENOEXEC		(neg 8))
(define-inline-constant ENOKEY		(neg 126))
(define-inline-constant ENOLCK		(neg 37))
(define-inline-constant ENOLINK		(neg 67))
(define-inline-constant ENOMEDIUM	(neg 123))
(define-inline-constant ENOMEM		(neg 12))
(define-inline-constant ENOMSG		(neg 42))
(define-inline-constant ENONET		(neg 64))
(define-inline-constant ENOPKG		(neg 65))
(define-inline-constant ENOPROTOOPT	(neg 92))
(define-inline-constant ENOSPC		(neg 28))
(define-inline-constant ENOSR		(neg 63))
(define-inline-constant ENOSTR		(neg 60))
(define-inline-constant ENOSYS		(neg 38))
(define-inline-constant ENOTBLK		(neg 15))
(define-inline-constant ENOTCONN	(neg 107))
(define-inline-constant ENOTDIR		(neg 20))
(define-inline-constant ENOTEMPTY	(neg 39))
(define-inline-constant ENOTNAM		(neg 118))
(define-inline-constant ENOTRECOVERABLE	(neg 131))
(define-inline-constant ENOTSOCK	(neg 88))
(define-inline-constant ENOTTY		(neg 25))
(define-inline-constant ENOTUNIQ	(neg 76))
(define-inline-constant ENXIO		(neg 6))
(define-inline-constant EOPNOTSUPP	(neg 95))
(define-inline-constant EOVERFLOW	(neg 75))
(define-inline-constant EOWNERDEAD	(neg 130))
(define-inline-constant EPERM		(neg 1))
(define-inline-constant EPFNOSUPPORT	(neg 96))
(define-inline-constant EPIPE		(neg 32))
(define-inline-constant EPROTO		(neg 71))
(define-inline-constant EPROTONOSUPPORT	(neg 93))
(define-inline-constant EPROTOTYPE	(neg 91))
(define-inline-constant ERANGE		(neg 34))
(define-inline-constant EREMCHG		(neg 78))
(define-inline-constant EREMOTE		(neg 66))
(define-inline-constant EREMOTEIO	(neg 121))
(define-inline-constant ERESTART	(neg 85))
(define-inline-constant EROFS		(neg 30))
(define-inline-constant ESHUTDOWN	(neg 108))
(define-inline-constant ESOCKTNOSUPPORT	(neg 94))
(define-inline-constant ESPIPE		(neg 29))
(define-inline-constant ESRCH		(neg 3))
(define-inline-constant ESRMNT		(neg 69))
(define-inline-constant ESTALE		(neg 116))
(define-inline-constant ESTRPIPE	(neg 86))
(define-inline-constant ETIME		(neg 62))
(define-inline-constant ETIMEDOUT	(neg 110))
(define-inline-constant ETOOMANYREFS	(neg 109))
(define-inline-constant ETXTBSY		(neg 26))
(define-inline-constant EUCLEAN		(neg 117))
(define-inline-constant EUNATCH		(neg 49))
(define-inline-constant EUSERS		(neg 87))
(define-inline-constant EWOULDBLOCK	(neg 11))
(define-inline-constant EXDEV		(neg 18))
(define-inline-constant EXFULL		(neg 54))


;;;; errno interface

(define-syntax errno-code
  (lambda (stx)
    (syntax-case stx ()
      ((_ ?code)
       (identifier? #'?code)
       (let ((sym (syntax->datum #'?code)))
	 (define them
	   '(E2BIG		EACCES		EADDRINUSE
	     EADDRNOTAVAIL	EADV		EAFNOSUPPORT
	     EAGAIN		EALREADY	EBADE
	     EBADF		EBADFD		EBADMSG
	     EBADR		EBADRQC		EBADSLT
	     EBFONT		EBUSY		ECANCELED
	     ECHILD		ECHRNG		ECOMM
	     ECONNABORTED	ECONNREFUSED	ECONNRESET
	     EDEADLK		EDEADLOCK	EDESTADDRREQ
	     EDOM		EDOTDOT		EDQUOT
	     EEXIST		EFAULT		EFBIG
	     EHOSTDOWN		EHOSTUNREACH	EIDRM
	     EILSEQ		EINPROGRESS	EINTR
	     EINVAL		EIO		EISCONN
	     EISDIR		EISNAM		EKEYEXPIRED
	     EKEYREJECTED	EKEYREVOKED	EL2HLT
	     EL2NSYNC		EL3HLT		EL3RST
	     ELIBACC		ELIBBAD		ELIBEXEC
	     ELIBMAX		ELIBSCN		ELNRNG
	     ELOOP		EMEDIUMTYPE	EMFILE
	     EMLINK		EMSGSIZE	EMULTIHOP
	     ENAMETOOLONG	ENAVAIL		ENETDOWN
	     ENETRESET		ENETUNREACH	ENFILE
	     ENOANO		ENOBUFS		ENOCSI
	     ENODATA		ENODEV		ENOENT
	     ENOEXEC		ENOKEY		ENOLCK
	     ENOLINK		ENOMEDIUM	ENOMEM
	     ENOMSG		ENONET		ENOPKG
	     ENOPROTOOPT	ENOSPC		ENOSR
	     ENOSTR		ENOSYS		ENOTBLK
	     ENOTCONN		ENOTDIR		ENOTEMPTY
	     ENOTNAM		ENOTRECOVERABLE	ENOTSOCK
	     ENOTTY		ENOTUNIQ	ENXIO
	     EOPNOTSUPP		EOVERFLOW	EOWNERDEAD
	     EPERM		EPFNOSUPPORT	EPIPE
	     EPROTO		EPROTONOSUPPORT	EPROTOTYPE
	     ERANGE		EREMCHG		EREMOTE
	     EREMOTEIO		ERESTART	EROFS
	     ESHUTDOWN		ESOCKTNOSUPPORT	ESPIPE
	     ESRCH		ESRMNT		ESTALE
	     ESTRPIPE		ETIME		ETIMEDOUT
	     ETOOMANYREFS	ETXTBSY		EUCLEAN
	     EUNATCH		EUSERS		EWOULDBLOCK
	     EXDEV		EXFULL))
	 (if (memq sym them)
	     (datum->syntax #'%memory-guardian sym)
	   (syntax-violation 'errno-code "invalid symbol as errno code" (syntax->datum stx) sym)))))))

(define-syntax case-errno
  (lambda (stx)
    (define who 'case-errno)
    (define them
      ;;I know that it is really  ugly to duplicate this list, but it is
      ;;also  much simpler  than  creating an  external  library for  it
      ;;(Marco Maggi; Nov 28, 2011).
      '( ;;
	E2BIG		EACCES		EADDRINUSE
	EADDRNOTAVAIL	EADV		EAFNOSUPPORT
	EAGAIN		EALREADY	EBADE
	EBADF		EBADFD		EBADMSG
	EBADR		EBADRQC		EBADSLT
	EBFONT		EBUSY		ECANCELED
	ECHILD		ECHRNG		ECOMM
	ECONNABORTED	ECONNREFUSED	ECONNRESET
	EDEADLK		EDEADLOCK	EDESTADDRREQ
	EDOM		EDOTDOT		EDQUOT
	EEXIST		EFAULT		EFBIG
	EHOSTDOWN	EHOSTUNREACH	EIDRM
	EILSEQ		EINPROGRESS	EINTR
	EINVAL		EIO		EISCONN
	EISDIR		EISNAM		EKEYEXPIRED
	EKEYREJECTED	EKEYREVOKED	EL2HLT
	EL2NSYNC	EL3HLT		EL3RST
	ELIBACC		ELIBBAD		ELIBEXEC
	ELIBMAX		ELIBSCN		ELNRNG
	ELOOP		EMEDIUMTYPE	EMFILE
	EMLINK		EMSGSIZE	EMULTIHOP
	ENAMETOOLONG	ENAVAIL		ENETDOWN
	ENETRESET	ENETUNREACH	ENFILE
	ENOANO		ENOBUFS		ENOCSI
	ENODATA		ENODEV		ENOENT
	ENOEXEC		ENOKEY		ENOLCK
	ENOLINK		ENOMEDIUM	ENOMEM
	ENOMSG		ENONET		ENOPKG
	ENOPROTOOPT	ENOSPC		ENOSR
	ENOSTR		ENOSYS		ENOTBLK
	ENOTCONN	ENOTDIR		ENOTEMPTY
	ENOTNAM		ENOTRECOVERABLE	ENOTSOCK
	ENOTTY		ENOTUNIQ	ENXIO
	EOPNOTSUPP	EOVERFLOW	EOWNERDEAD
	EPERM		EPFNOSUPPORT	EPIPE
	EPROTO		EPROTONOSUPPORT	EPROTOTYPE
	ERANGE		EREMCHG		EREMOTE
	EREMOTEIO	ERESTART	EROFS
	ESHUTDOWN	ESOCKTNOSUPPORT	ESPIPE
	ESRCH		ESRMNT		ESTALE
	ESTRPIPE	ETIME		ETIMEDOUT
	ETOOMANYREFS	ETXTBSY		EUCLEAN
	EUNATCH		EUSERS		EWOULDBLOCK
	EXDEV		EXFULL))
    (define (%codes->bindings x)
      (let ((context #'EPERM))
	(map (lambda (list-of-symbols)
	       (map (lambda (symbol)
		      (cond ((not (symbol? symbol))
			     (syntax-violation who "expected symbol as symbolic errno code" symbol))
			    ((memq symbol them)
			     (datum->syntax context symbol))
			    (else
			     (syntax-violation who "unknown symbolic errno code" symbol))))
		 list-of-symbols))
	  (syntax->datum x))))
    (syntax-case stx (else)
      ((_ ?errno ((?code0 ?code ...) . ?body) ... (else . ?else-body))
       (with-syntax ((((CODE0 CODE ...) ...) (%codes->bindings #'((?code0 ?code ...) ...))))
	 #'(let ((errno ?errno))
	     (cond ((or (and (fixnum? CODE0) ($fx= errno CODE0))
			(and (fixnum? CODE)  ($fx= errno CODE))
			...)
		    . ?body)
		   ...
		   (else . ?else-body)))))
      ((_ ?errno ((?code0 ?code ...) . ?body) ...)
       (with-syntax ((((CODE0 CODE ...) ...) (%codes->bindings #'((?code0 ?code ...) ...))))
	 #'(let ((errno ?errno))
	     (cond ((or (and (fixnum? CODE0) ($fx= errno CODE0))
			(and (fixnum? CODE)  ($fx= errno CODE))
			...)
		    . ?body)
		   ...
		   (else
		    (assertion-violation #f "unknown errno code" errno))))))
      )))


;;;; done

)

;;; end of file
